@@ -1,5 +1,6 @@
 #!/usr/bin/env ruby -w
 # encoding: UTF-8
+# frozen_string_literal: false
 
 # tc_features.rb
 #
@@ -74,6 +75,14 @@ class TestCSV::Features < TestCSV
     end
   end
 
+  def test_bug_8405
+    TEST_CASES.each do |test_case|
+      assert_equal( test_case.last.map { |t| t.tr('"', "|") unless t.nil? },
+                    CSV.parse_line( test_case.first.tr('"', "|"),
+                                    quote_char: "|" ) )
+    end
+  end
+
   def test_csv_char_readers
     %w[col_sep row_sep quote_char].each do |reader|
       csv = CSV.new("abc,def", reader.to_sym => "|")
@@ -115,7 +124,9 @@ class TestCSV::Features < TestCSV
   end
 
   def test_unknown_options
-    assert_raise(ArgumentError) { CSV.new(String.new, unknown: :error) }
+    assert_raise_with_message(ArgumentError, /unknown/) {
+      CSV.new(@sample_data, unknown: :error)
+    }
   end
 
   def test_skip_blanks
@@ -131,13 +142,35 @@ class TestCSV::Features < TestCSV
     assert_equal(3, count)
   end
 
+  def test_liberal_parsing
+    input = '"Johnson, Dwayne",Dwayne "The Rock" Johnson'
+    assert_raise(CSV::MalformedCSVError) do
+        CSV.parse_line(input)
+    end
+    assert_equal(["Johnson, Dwayne", 'Dwayne "The Rock" Johnson'],
+                 CSV.parse_line(input, liberal_parsing: true))
+
+    input = '"quoted" field'
+    assert_raise(CSV::MalformedCSVError) do
+        CSV.parse_line(input)
+    end
+    assert_equal(['"quoted" field'],
+                 CSV.parse_line(input, liberal_parsing: true))
+
+    assert_raise(CSV::MalformedCSVError) do
+      CSV.parse_line('is,this "three," or four,fields', liberal_parsing: true)
+    end
+
+    assert_equal(["is", 'this "three', ' or four"', "fields"],
+      CSV.parse_line('is,this "three, or four",fields', liberal_parsing: true))
+  end
+
   def test_csv_behavior_readers
     %w[ unconverted_fields return_headers write_headers
         skip_blanks        force_quotes ].each do |behavior|
-      assert( !CSV.new("abc,def").send("#{behavior}?"),
-              "Behavior defaulted to on." )
+      assert_not_predicate(CSV.new("abc,def"), "#{behavior}?", "Behavior defaulted to on.")
       csv = CSV.new("abc,def", behavior.to_sym => true)
-      assert(csv.send("#{behavior}?"), "Behavior change now registered.")
+      assert_predicate(csv, "#{behavior}?", "Behavior change now registered.")
     end
   end
 
@@ -184,9 +217,9 @@ class TestCSV::Features < TestCSV
   # reported by Chris Roos
   def test_failing_to_reset_headers_in_rewind_bug_fix
     csv = CSV.new("forename,surname", headers: true, return_headers: true)
-    csv.each { |row| assert row.header_row? }
+    csv.each {|row| assert_predicate row, :header_row?}
     csv.rewind
-    csv.each { |row| assert row.header_row? }
+    csv.each {|row| assert_predicate row, :header_row?}
   end
 
   # reported by Dave Burt
@@ -209,6 +242,8 @@ class TestCSV::Features < TestCSV
                )
     end
     assert_equal("\r\n", zipped.row_sep)
+  ensure
+    zipped.close
   end if defined?(Zlib::GzipReader)
 
   def test_gzip_writer_bug_fix
@@ -223,25 +258,24 @@ class TestCSV::Features < TestCSV
       zipped << [1, 2, 3]
       zipped.close
 
-      assert( Zlib::GzipReader.open(file) { |f| f.read }.
-                               include?($INPUT_RECORD_SEPARATOR),
-              "@row_sep did not default" )
+      assert_include(Zlib::GzipReader.open(file) {|f| f.read},
+                     $INPUT_RECORD_SEPARATOR, "@row_sep did not default")
     }
   end if defined?(Zlib::GzipWriter)
 
   def test_inspect_is_smart_about_io_types
     str = CSV.new("string,data").inspect
-    assert(str.include?("io_type:StringIO"), "IO type not detected.")
+    assert_include(str, "io_type:StringIO", "IO type not detected.")
 
     str = CSV.new($stderr).inspect
-    assert(str.include?("io_type:$stderr"), "IO type not detected.")
+    assert_include(str, "io_type:$stderr", "IO type not detected.")
 
     Tempfile.create(%w"temp .csv") {|tempfile|
       tempfile.close
       path = tempfile.path
       File.open(path, "w") { |csv| csv << "one,two,three\n1,2,3\n" }
       str  = CSV.open(path) { |csv| csv.inspect }
-      assert(str.include?("io_type:File"), "IO type not detected.")
+      assert_include(str, "io_type:File", "IO type not detected.")
     }
   end
 
@@ -254,7 +288,7 @@ class TestCSV::Features < TestCSV
 
   def test_inspect_shows_headers_when_available
     CSV.new("one,two,three\n1,2,3\n", headers: true) do |csv|
-      assert(csv.inspect.include?("headers:true"), "Header hint not shown.")
+      assert_include(csv.inspect, "headers:true", "Header hint not shown.")
       csv.shift  # load headers
       assert_match(/headers:\[[^\]]+\]/, csv.inspect)
     end
@@ -262,28 +296,28 @@ class TestCSV::Features < TestCSV
 
   def test_inspect_encoding_is_ascii_compatible
     CSV.new("one,two,three\n1,2,3\n".encode("UTF-16BE")) do |csv|
-      assert( Encoding.compatible?( Encoding.find("US-ASCII"),
-                                    csv.inspect.encoding ),
-              "inspect() was not ASCII compatible." )
+      assert_send([Encoding, :compatible?,
+                   Encoding.find("US-ASCII"), csv.inspect.encoding],
+                  "inspect() was not ASCII compatible.")
     end
   end
 
   def test_version
     assert_not_nil(CSV::VERSION)
     assert_instance_of(String, CSV::VERSION)
-    assert(CSV::VERSION.frozen?)
+    assert_predicate(CSV::VERSION, :frozen?)
     assert_match(/\A\d\.\d\.\d\Z/, CSV::VERSION)
   end
 
   def test_accepts_comment_skip_lines_option
     assert_nothing_raised(ArgumentError) do
-      CSV.new nil, :skip_lines => /\A\s*#/
+      CSV.new(@sample_data, :skip_lines => /\A\s*#/)
     end
   end
 
   def test_accepts_comment_defaults_to_nil
-    c = CSV.new nil
-    assert_equal c.skip_lines, nil
+    c = CSV.new(@sample_data)
+    assert_nil(c.skip_lines)
   end
 
   class RegexStub
@@ -291,8 +325,8 @@ class TestCSV::Features < TestCSV
 
   def test_requires_skip_lines_to_call_match
     regex_stub = RegexStub.new
-    assert_raise(ArgumentError) do
-      CSV.new nil, :skip_lines => regex_stub
+    assert_raise_with_message(ArgumentError, /skip_lines/) do
+      CSV.new(@sample_data, :skip_lines => regex_stub)
     end
   end
 
